@@ -18,6 +18,12 @@
 Wraps the GLPK solver by subclassing and extending :class:`Model`,
 :class:`Variable`, and :class:`Constraint` from :mod:`interface`.
 """
+import six
+
+if six.PY3:
+    from io import StringIO
+else:
+    import StringIO
 import copy
 import sys
 
@@ -105,7 +111,7 @@ _CPLEX_VTYPE_TO_VTYPE = {'C': 'continuous', 'I': 'integer', 'B': 'binary'}
 # FIXME: what about 'S': 'semi_continuous', 'N': 'semi_integer'
 
 _VTYPE_TO_CPLEX_VTYPE = dict(
-    [(val, key) for key, val in _CPLEX_VTYPE_TO_VTYPE.iteritems()]
+    [(val, key) for key, val in six.iteritems(_CPLEX_VTYPE_TO_VTYPE)]
 )
 
 
@@ -166,15 +172,7 @@ class Variable(interface.Variable):
     def primal(self):
         if self.problem:
             primal_from_solver = self.problem.problem.solution.get_values(self.name)
-            if primal_from_solver >= self.lb and primal_from_solver <= self.ub:
-                return primal_from_solver
-            else:
-                if (self.lb - primal_from_solver) <= 1e-6:
-                    return self.lb
-                elif (self.ub - primal_from_solver) >= -1e-6:
-                    return self.ub
-                else:
-                    raise AssertionError('The primal value %s returned by the solver is out of bounds for variable %s (lb=%s, ub=%s)' % (primal_from_solver, self.name, self.lb, self.ub))
+            return self.__round_primal_to_bounds(primal_from_solver)
         else:
             return None
 
@@ -205,7 +203,7 @@ class Constraint(interface.Constraint):
     def _set_coefficients_low_level(self, variables_coefficients_dict):
         self_name = self.name
         if self.is_Linear:
-            cplex_format = [(self_name, variable.name, coefficient) for variable, coefficient in variables_coefficients_dict.iteritems()]
+            cplex_format = [(self_name, variable.name, coefficient) for variable, coefficient in six.iteritems(variables_coefficients_dict)]
             self.problem.problem.linear_constraints.set_coefficients(cplex_format)
         else:
             raise Exception('_set_coefficients_low_level works only with linear constraints in the cplex interface.')
@@ -309,7 +307,7 @@ class Configuration(interface.MathematicalProgrammingConfiguration):
 
     def __setstate__(self, state):
         self.__init__()
-        for key, val in state.iteritems():
+        for key, val in six.iteritems(state):
             setattr(self, key, val)
 
     @property
@@ -334,23 +332,55 @@ class Configuration(interface.MathematicalProgrammingConfiguration):
 
     @verbosity.setter
     def verbosity(self, value):
+
+        class StreamHandler(StringIO.StringIO):
+
+            def __init__(self, logger, *args, **kwargs):
+                StringIO.StringIO.__init__(self, *args, **kwargs)
+                self.logger = logger
+
+        class ErrorStreamHandler(StreamHandler):
+
+            def flush(self):
+                self.logger.error(self.getvalue())
+
+        class WarningStreamHandler(StreamHandler):
+
+            def flush(self):
+                self.logger.warn(self.getvalue())
+
+        class LogStreamHandler(StreamHandler):
+
+            def flush(self):
+                self.logger.debug(self.getvalue())
+
+        class ResultsStreamHandler(StreamHandler):
+
+            def flush(self):
+                self.logger.debug(self.getvalue())
+
+        logger = logging.getLogger()
+        error_stream_handler = ErrorStreamHandler(logger)
+        warning_stream_handler = WarningStreamHandler(logger)
+        log_stream_handler = LogStreamHandler(logger)
+        results_stream_handler = LogStreamHandler(logger)
         if self.problem is not None:
             problem = self.problem.problem
             if value == 0:
-                problem.set_error_stream(None)
-                problem.set_warning_stream(None)
-                problem.set_log_stream(None)
-                problem.set_results_stream(None)
+                problem.set_error_stream(error_stream_handler)
+                problem.set_warning_stream(warning_stream_handler)
+                problem.set_log_stream(log_stream_handler)
+                problem.set_results_stream(results_stream_handler)
             elif value == 1:
                 problem.set_error_stream(sys.stderr)
-                problem.set_warning_stream(None)
-                problem.set_log_stream(None)
-                problem.set_results_stream(None)
+                problem.set_warning_stream(warning_stream_handler)
+                problem.set_log_stream(log_stream_handler)
+                problem.set_results_stream(results_stream_handler)
             elif value == 2:
                 problem.set_error_stream(sys.stderr)
                 problem.set_warning_stream(sys.stderr)
-                problem.set_log_stream(None)
-                problem.set_results_stream(None)
+                problem.set_log_stream(log_stream_handler)
+                problem.set_results_stream(results_stream_handler)
             elif value == 3:
                 problem.set_error_stream(sys.stderr)
                 problem.set_warning_stream(sys.stderr)
@@ -421,7 +451,7 @@ class Model(interface.Model):
                     else:
                         constr = Constraint(lhs, lb=rhs + range_val, ub=rhs, name=name, problem=self)
                 else:
-                    raise Exception, '%s is not a recognized constraint sense.' % sense
+                    raise Exception('%s is not a recognized constraint sense.' % sense)
 
                 for variable in constraint_variables:
                     try:
@@ -442,18 +472,22 @@ class Model(interface.Model):
                 name=self.problem.objective.get_name()
             )
         else:
-            raise Exception, "Provided problem is not a valid CPLEX model."
+            raise Exception("Provided problem is not a valid CPLEX model.")
         self.configuration = Configuration(problem=self, verbosity=0)
 
     def __getstate__(self):
-        cplex_repr = self.__repr__()
-        repr_dict = {'cplex_repr': cplex_repr}
+        tmp_file = tempfile.mktemp(suffix=".sav")
+        self.problem.write(tmp_file)
+        cplex_binary = open(tmp_file).read()
+        repr_dict = {'cplex_binary': cplex_binary, 'status': self.status}
         return repr_dict
 
     def __setstate__(self, repr_dict):
         tmp_file = tempfile.mktemp(suffix=".sav")
-        open(tmp_file, 'w').write(repr_dict['cplex_repr'])
+        open(tmp_file, 'w').write(repr_dict['cplex_binary'])
         problem = cplex.Cplex(tmp_file)
+        if repr_dict['status'] == 'optimal':
+            problem.solve()  # since the start is an optimal solution, nothing will happen here
         self.__init__(problem=problem)
 
     @property
@@ -463,10 +497,10 @@ class Model(interface.Model):
     @objective.setter
     def objective(self, value):
         super(Model, self.__class__).objective.fset(self, value)
-        for i in xrange(len(self.problem.objective.get_linear())):
+        for i in range(len(self.problem.objective.get_linear())):
             self.problem.objective.set_linear(i, 0.)
         expression = self._objective.expression
-        if isinstance(expression, types.FloatType) or isinstance(expression, types.IntType) or expression.is_Number:
+        if isinstance(expression, float) or isinstance(expression, int) or expression.is_Number:
             pass
         else:
             if expression.is_Symbol:
@@ -494,12 +528,6 @@ class Model(interface.Model):
         cplex_form = open(tmp_file).read()
         return cplex_form
 
-    def __repr__(self):
-        tmp_file = tempfile.mktemp(suffix=".sav")
-        self.problem.write(tmp_file)
-        cplex_form = open(tmp_file).read()
-        return cplex_form
-
     def optimize(self):
         self.problem.solve()
         cplex_status = self.problem.solution.get_status()
@@ -511,7 +539,7 @@ class Model(interface.Model):
         if not translation: translation = {'E': '==', 'L': '<', 'G': '>'}
         try:
             return translation[sense]
-        except KeyError, e:
+        except KeyError as e:
             raise Exception(' '.join(('Sense', sense, 'is not a proper relational operator, e.g. >, <, == etc.')))
 
     def _add_variable(self, variable):
@@ -594,13 +622,13 @@ if __name__ == '__main__':
     model = Model(name='Simple model')
     model.objective = obj
     model.add([c1, c2, c3])
-    print model
+    print(model)
     status = model.optimize()
-    print "status:", model.status
-    print "objective value:", model.objective.value
+    print("status:", model.status)
+    print("objective value:", model.objective.value)
 
-    for var_name, var in model.variables.iteritems():
-        print var_name, "=", var.primal
+    for var_name, var in model.variables.items():
+        print(var_name, "=", var.primal)
 
 
         # from cplex import Cplex
