@@ -22,6 +22,7 @@ Wraps the GLPK solver by subclassing and extending :class:`Model`,
 
 import logging
 import sys
+import six
 
 import types
 
@@ -43,9 +44,9 @@ from swiglpk import glp_find_col, glp_get_col_prim, glp_get_col_dual, GLP_CV, GL
     glp_set_mat_row, glp_set_col_bnds, glp_set_row_bnds, GLP_FR, GLP_UP, GLP_LO, GLP_FX, GLP_DB, glp_del_rows, \
     glp_get_mat_row, glp_get_row_ub, glp_get_row_type, glp_get_row_lb, glp_get_row_name, glp_get_obj_coef, \
     glp_get_obj_dir, glp_scale_prob, GLP_SF_AUTO, glp_get_num_int, glp_get_num_bin, glp_version, glp_mip_col_val, \
-    glp_mip_obj_val
+    glp_mip_obj_val, glp_mip_status, GLP_ETMLIM
 
-import interface
+from optlang import interface
 
 
 _GLPK_STATUS_TO_STATUS = {
@@ -64,7 +65,7 @@ _GLPK_VTYPE_TO_VTYPE = {
 }
 
 _VTYPE_TO_GLPK_VTYPE = dict(
-    [(val, key) for key, val in _GLPK_VTYPE_TO_VTYPE.iteritems()]
+    [(val, key) for key, val in six.iteritems(_GLPK_VTYPE_TO_VTYPE)]
 )
 
 
@@ -77,7 +78,7 @@ class Variable(interface.Variable):
     @property
     def index(self):
         try:
-            i = glp_find_col(self.problem.problem, self.name)
+            i = glp_find_col(self.problem.problem, str(self.name))
             if i != 0:
                 return i
             else:
@@ -116,15 +117,7 @@ class Variable(interface.Variable):
                 primal_from_solver = glp_mip_col_val(self.problem.problem, self.index)
             else:
                 raise TypeError("Unknown variable type")
-            if (primal_from_solver >= self.lb or self.lb is None) and (primal_from_solver <= self.ub or self.ub is None):
-                return primal_from_solver
-            else:
-                if (self.lb - primal_from_solver) <= 1e-6:
-                    return self.lb
-                elif (self.ub - primal_from_solver) >= -1e-6:
-                    return self.ub
-                else:
-                    raise AssertionError('The primal value %s returned by the solver is out of bounds for variable %s (lb=%s, ub=%s)' % (primal_from_solver, self.name, self.lb, self.ub))
+            return self.__round_primal_to_bounds(primal_from_solver)
         else:
             return None
 
@@ -143,7 +136,7 @@ class Variable(interface.Variable):
         super(Variable, self).__setattr__(name, value)
         if getattr(self, 'problem', None):
             if name == 'name':
-                glp_set_col_name(self.problem.problem, glp_find_col(self.problem.problem, old_name), value)
+                glp_set_col_name(self.problem.problem, glp_find_col(self.problem.problem, old_name), str(value))
 
 
 class Constraint(interface.Constraint):
@@ -175,7 +168,7 @@ class Constraint(interface.Constraint):
     def _set_coefficients_low_level(self, variables_coefficients_dict):
         if self.problem is not None:
             problem = self.problem.problem
-            indices_coefficients_dict = dict([(variable.index, coefficient) for variable, coefficient in variables_coefficients_dict.iteritems()])
+            indices_coefficients_dict = dict([(variable.index, coefficient) for variable, coefficient in six.iteritems(variables_coefficients_dict)])
             num_cols = glp_get_num_cols(problem)
             ia = intArray(num_cols + 1)
             da = doubleArray(num_cols + 1)
@@ -186,7 +179,7 @@ class Constraint(interface.Constraint):
                     da[i] = indices_coefficients_dict[ia[i]]
                 except KeyError:
                     pass
-            print glp_set_mat_row(self.problem.problem, index, num, ia, da)
+            print(glp_set_mat_row(self.problem.problem, index, num, ia, da))
         else:
             raise Exception('_set_coefficients_low_level works only if a constraint is associated with a solver instance.')
 
@@ -206,7 +199,7 @@ class Constraint(interface.Constraint):
     @property
     def index(self):
         try:
-            i = glp_find_row(self.problem.problem, self.name)
+            i = glp_find_row(self.problem.problem, str(self.name))
             if i != 0:
                 return i
             else:
@@ -250,7 +243,7 @@ class Constraint(interface.Constraint):
         super(Constraint, self).__setattr__(name, value)
         if getattr(self, 'problem', None):
             if name == 'name':
-                glp_set_row_name(self.problem.problem, glp_find_row(self.problem.problem, old_name), value)
+                glp_set_row_name(self.problem.problem, glp_find_row(self.problem.problem, old_name), str(value))
             elif name == 'lb' or name == 'ub':
                 self.problem._glpk_set_row_bounds(self)
 
@@ -302,7 +295,7 @@ class Objective(interface.Objective):
         if self.problem is not None:
             variables = self.problem.variables
             def term_generator():
-                for index in xrange(1, glp_get_num_cols(self.problem.problem) + 1):
+                for index in range(1, glp_get_num_cols(self.problem.problem) + 1):
                     coeff = glp_get_obj_coef(self.problem.problem, index)
                     if coeff != 0.:
                         yield (sympy.RealNumber(coeff), variables[index - 1])
@@ -365,18 +358,20 @@ class Configuration(interface.MathematicalProgrammingConfiguration):
         self._iocp = glp_iocp()
         glp_init_smcp(self._smcp)
         glp_init_iocp(self._iocp)
+        self._max_time = min(self._smcp.tm_lim, self._iocp.tm_lim)
         self._set_presolve(presolve)
         self._presolve = presolve
         self._set_verbosity(verbosity)
         self._verbosity = verbosity
-        self._timeout = None
+        self._set_timeout(timeout)
+        self._timeout = timeout
 
     def __getstate__(self):
         return {'presolve': self.presolve, 'verbosity': self.verbosity, 'timeout': self.timeout}
 
     def __setstate__(self, state):
         self.__init__()
-        for key, val in state.iteritems():
+        for key, val in six.iteritems(state):
             setattr(self, key, val)
 
     def _set_presolve(self, value):
@@ -406,6 +401,14 @@ class Configuration(interface.MathematicalProgrammingConfiguration):
                 % value
             )
 
+    def _set_timeout(self, value):
+        if value is None:
+            self._smcp.tm_lim = self._max_time
+            self._iocp.tm_lim = self._max_time
+        else:
+            self._smcp.tm_lim = value * 1000  # milliseconds to seconds
+            self._iocp.tm_lim = value * 1000
+
     @property
     def presolve(self):
         return self._presolve
@@ -430,12 +433,8 @@ class Configuration(interface.MathematicalProgrammingConfiguration):
 
     @timeout.setter
     def timeout(self, value):
-        if value is None:
-            self._smcp.tm_lim = sys.maxint
-            self._iocp.tm_lim = sys.maxint
-        else:
-            self._smcp.tm_lim = value * 1000  # milliseconds to seconds
-            self._iocp.tm_lim = value * 1000
+        self._set_timeout(value)
+        self._timeout = value
 
 class Model(interface.Model):
     """GLPK solver interface"""
@@ -450,7 +449,7 @@ class Model(interface.Model):
             self.problem = glp_create_prob()
             glp_create_index(self.problem)
             if self.name is not None:
-                glp_set_prob_name(self.problem, self.name)
+                glp_set_prob_name(self.problem, str(self.name))
 
         else:
             try:
@@ -460,7 +459,7 @@ class Model(interface.Model):
                 raise TypeError("Provided problem is not a valid GLPK model.")
             row_num = glp_get_num_rows(self.problem)
             col_num = glp_get_num_cols(self.problem)
-            for i in xrange(1, col_num + 1):
+            for i in range(1, col_num + 1):
                 var = Variable(
                     glp_get_col_name(self.problem, i),
                     lb=glp_get_col_lb(self.problem, i),
@@ -473,7 +472,7 @@ class Model(interface.Model):
                 super(Model, self)._add_variable(var)
             variables = self.variables
 
-            for j in xrange(1, row_num + 1):
+            for j in range(1, row_num + 1):
                 ia = intArray(col_num + 1)
                 da = doubleArray(col_num + 1)
                 nnz = glp_get_mat_row(self.problem, j, ia, da)
@@ -518,7 +517,7 @@ class Model(interface.Model):
 
             term_generator = (
                 (glp_get_obj_coef(self.problem, index), variables[index - 1])
-                for index in xrange(1, glp_get_num_cols(problem) + 1)
+                for index in range(1, glp_get_num_cols(problem) + 1)
             )
             self._objective = Objective(
                 _unevaluated_Add(
@@ -532,7 +531,7 @@ class Model(interface.Model):
 
     def __getstate__(self):
         glpk_repr = self._glpk_representation()
-        repr_dict = {'glpk_repr': glpk_repr}
+        repr_dict = {'glpk_repr': glpk_repr, 'glpk_status': self.status}
         return repr_dict
 
     def __setstate__(self, repr_dict):
@@ -541,6 +540,8 @@ class Model(interface.Model):
         problem = glp_create_prob()
         glp_read_prob(problem, 0, tmp_file)
         self.__init__(problem=problem)
+        if repr_dict['glpk_status'] == 'optimal':
+            self.optimize()  # since the start is an optimal solution, nothing will happen here
 
     def __copy__(self):
         return Model(problem=self.problem)
@@ -563,7 +564,7 @@ class Model(interface.Model):
                     glp_set_obj_coef(self.problem, variable.index, 0.)
         super(Model, self.__class__).objective.fset(self, value)
         expression = self._objective._expression
-        if isinstance(expression, types.FloatType) or isinstance(expression, types.IntType) or expression.is_Number:
+        if isinstance(expression, float) or isinstance(expression, int) or expression.is_Number:
             pass
         else:
             if expression.is_Symbol:
@@ -597,33 +598,59 @@ class Model(interface.Model):
         glpk_form = open(tmp_file).read()
         return glpk_form
 
+    def _run_glp_simplex(self):
+        return_value = glp_simplex(self.problem, self.configuration._smcp)
+        glpk_status = glp_get_status(self.problem)
+        if return_value == 0:
+            status = _GLPK_STATUS_TO_STATUS[glpk_status]
+        elif return_value == GLP_ETMLIM:
+            status = interface.TIME_LIMIT
+        else:
+            status = _GLPK_STATUS_TO_STATUS[glpk_status]
+            if status == interface.UNDEFINED:
+                log.debug("Status undefined. GLPK status code returned by glp_simplex was %d" % return_value)
+        return status
+
+    def _run_glp_mip(self):
+        return_value = glp_intopt(self.problem, self.configuration._iocp)
+        glpk_status = glp_mip_status(self.problem)
+        if return_value == 0:
+            status = _GLPK_STATUS_TO_STATUS[glpk_status]
+        elif return_value == GLP_ETMLIM:
+            status = interface.TIME_LIMIT
+        else:
+            status = _GLPK_STATUS_TO_STATUS[glpk_status]
+            if status == interface.UNDEFINED:
+                log.debug("Status undefined. GLPK status code returned by glp_intopt was %d" % return_value)
+        return status
+
     def optimize(self):
-        glp_simplex(self.problem, self.configuration._smcp)
-        status = _GLPK_STATUS_TO_STATUS[glp_get_status(self.problem)]
-        if status == 'undefined' or status == 'infeasible':
+        status = self._run_glp_simplex()
+
+        if status == interface.UNDEFINED or status == interface.INFEASIBLE:
+            # Let's see if the presolver and some scaling can fix this issue
             glp_scale_prob(self.problem, GLP_SF_AUTO)
             original_presolve_setting = self.configuration.presolve
             self.configuration.presolve = True
-            glp_simplex(self.problem, self.configuration._smcp)
+            status = self._run_glp_simplex()
             self.configuration.presolve = original_presolve_setting
         if (glp_get_num_int(self.problem) + glp_get_num_bin(self.problem)) > 0:
-            glp_intopt(self.problem, self.configuration._iocp)
-            status = _GLPK_STATUS_TO_STATUS[glp_get_status(self.problem)]
+            status = self._run_glp_mip()
             if status == 'undefined' or status == 'infeasible':
+                # Let's see if the presolver and some scaling can fix this issue
                 glp_scale_prob(self.problem, GLP_SF_AUTO)
                 original_presolve_setting = self.configuration.presolve
                 self.configuration.presolve = True
-                glp_intopt(self.problem, self.configuration._iocp)
+                status = self._run_glp_mip()
                 self.configuration.presolve = original_presolve_setting
-        glpk_status = glp_get_status(self.problem)
-        self._status = _GLPK_STATUS_TO_STATUS[glpk_status]
-        return self.status
+        self._status = status
+        return status
 
     def _add_variable(self, variable):
         super(Model, self)._add_variable(variable)
         glp_add_cols(self.problem, 1)
         index = glp_get_num_cols(self.problem)
-        glp_set_col_name(self.problem, index, variable.name)
+        glp_set_col_name(self.problem, index, str(variable.name))
         variable.problem = self
         self._glpk_set_col_bounds(variable)
         glp_set_col_kind(self.problem, variable.index, _VTYPE_TO_GLPK_VTYPE[variable.type])
@@ -634,7 +661,7 @@ class Model(interface.Model):
 
             if len(variables) > 350:
                 delete_indices = [variable.index - 1 for variable in variables]
-                keep_indices = [i for i in xrange(0, len(self.variables)) if i not in delete_indices]
+                keep_indices = [i for i in range(0, len(self.variables)) if i not in delete_indices]
                 self._variables = self.variables.fromkeys(keep_indices)
             else:
                 for variable in variables:
@@ -662,7 +689,7 @@ class Model(interface.Model):
         constraint._problem = None  # This needs to be dones in order to not trigger constraint._get_expression()
         glp_add_rows(self.problem, 1)
         index = glp_get_num_rows(self.problem)
-        glp_set_row_name(self.problem, index, constraint.name)
+        glp_set_row_name(self.problem, index, str(constraint.name))
         num_cols = glp_get_num_cols(self.problem)
         index_array = intArray(num_cols + 1)
         value_array = doubleArray(num_cols + 1)
@@ -786,13 +813,13 @@ if __name__ == '__main__':
     model.objective = obj
     model.add([c1, c2, c3])
     status = model.optimize()
-    print "status:", model.status
-    print "objective value:", model.objective.value
+    print("status:", model.status)
+    print("objective value:", model.objective.value)
 
-    for var_name, var in model.variables.iteritems():
-        print var_name, "=", var.primal
+    for var_name, var in model.variables.items():
+        print(var_name, "=", var.primal)
 
-    print model
+    print(model)
 
     from swiglpk import glp_read_lp
 
@@ -800,17 +827,17 @@ if __name__ == '__main__':
     glp_read_lp(problem, None, "../tests/data/model.lp")
 
     solver = Model(problem=problem)
-    print solver.optimize()
-    print solver.objective
+    print(solver.optimize())
+    print(solver.objective)
 
     import time
 
     t1 = time.time()
-    print "pickling"
+    print("pickling")
     pickle_string = pickle.dumps(solver)
     resurrected_solver = pickle.loads(pickle_string)
     t2 = time.time()
-    print "Execution time: %s" % (t2 - t1)
+    print("Execution time: %s" % (t2 - t1))
 
     resurrected_solver.optimize()
-    print "Halelujah!", resurrected_solver.objective.value
\ No newline at end of file
+    print("Halelujah!", resurrected_solver.objective.value)
\ No newline at end of file
