@@ -22,6 +22,7 @@ extended for individual solvers.
 import logging
 import random
 import uuid
+import six
 
 import types
 import collections
@@ -107,7 +108,7 @@ class Variable(sympy.Symbol):
                 raise ValueError(
                     'Variable names cannot contain whitespace characters. "%s" contains whitespace character "%s".' % (
                         name, char))
-        super(Variable, self).__init__(name, *args, **kwargs)
+        sympy.Symbol.__init__(name, *args, **kwargs)  #TODO: change this back to use super
         self._lb = lb
         self._ub = ub
         self._type = type
@@ -169,7 +170,6 @@ class Variable(sympy.Symbol):
             raise ValueError(
                 "'%s' is not a valid variable type. Choose between 'continuous, 'integer', or 'binary'." % value)
 
-
     def __str__(self):
         """Print a string representation of variable.
 
@@ -198,6 +198,17 @@ class Variable(sympy.Symbol):
     def __setstate__(self, state):
         self.__dict__ = state
 
+    def __round_primal_to_bounds(self, primal, tolerance=1e-6):
+        if (primal >= self.lb or self.lb is None) and (primal <= self.ub or self.ub is None):
+                return primal
+        else:
+            if (primal <= self.lb) and ((self.lb - primal) <= tolerance):
+                return self.lb
+            elif (primal >= self.ub) and ((self.ub - primal) >= -tolerance):
+                return self.ub
+            else:
+                raise AssertionError('The primal value %s returned by the solver is out of bounds for variable %s (lb=%s, ub=%s)' % (primal, self.name, self.lb, self.ub))
+
 
 # noinspection PyPep8Naming
 class OptimizationExpression(object):
@@ -258,9 +269,9 @@ class OptimizationExpression(object):
         return self.expression.atoms(sympy.Symbol)
 
     def _canonicalize(self, expression):
-        if isinstance(expression, types.FloatType):
+        if isinstance(expression, float):
             return sympy.RealNumber(expression)
-        elif isinstance(expression, types.IntType):
+        elif isinstance(expression, int):
             return sympy.Integer(expression)
         else:
             return expression
@@ -535,9 +546,8 @@ class Model(object):
             for atom in value.expression.atoms(sympy.Symbol):
                 if isinstance(atom, Variable) and (atom.problem is None or atom.problem != self):
                     self._add_variable(atom)
-        except AttributeError, e:
-            if isinstance(value.expression, types.FunctionType) or isinstance(value.expression,
-                                                                              types.FloatType):
+        except AttributeError as e:
+            if isinstance(value.expression, six.types.FunctionType) or isinstance(value.expression, float):
                 pass
             else:
                 raise AttributeError(e)
@@ -663,11 +673,10 @@ class Model(object):
             "You're using the high level interface to optlang. Problems cannot be optimized in this mode. Choose from one of the solver specific interfaces.")
 
     def _add_variable(self, variable):
-        if self.variables.has_key(variable.name):
-            raise Exception('Model already contains a variable with name %s' % variable.name)
+        self.variables.append(variable)
         self._variables_to_constraints_mapping[variable.name] = set([])
         variable.problem = self
-        self.variables.append(variable)
+
         return variable
 
     def _remove_variables(self, variables):
@@ -699,7 +708,7 @@ class Model(object):
         constraint_id = constraint.name
         if sloppy is False:
             for var in constraint.variables:
-                if var.name not in self.variables:
+                if var.problem is not self:
                     self._add_variable(var)
                 try:
                     self._variables_to_constraints_mapping[var.name].add(constraint_id)
@@ -749,11 +758,11 @@ if __name__ == '__main__':
 
     try:
         sol = model.optimize()
-    except NotImplementedError, e:
-        print e
+    except NotImplementedError as e:
+        print(e)
 
-    print model
-    print model.variables
+    print(model)
+    print(model.variables)
 
     # model.remove(x1)
 
